import { ItemStack, Player, system, world,Container } from "@minecraft/server";


//rocket jump
world.beforeEvents.entityRemove.subscribe((event) => {
    const entity = event.removedEntity;

    if (entity.typeId == "minecraft:fireworks_rocket") {
        const knockbackStrength = 0.8; // Set the desired knockback strength

        // Get the explosion location
        const explosionLocation = entity.location;

        // Apply knockback to entities near the explosion
        const near = entity.dimension.getEntities({ location: explosionLocation, maxDistance: 6,excludeTypes:["minecraft:fireworks_rocket"]});

        for (const mod of near) {
                if(mod.isValid()){
                // Calculate the direction from the explosion to the entity
                const directionX = mod.location.x - explosionLocation.x;
                const directionZ = mod.location.z - explosionLocation.z;

                // Calculate the length of the direction vector
                const length = Math.sqrt(directionX * directionX + directionZ * directionZ);

                // Normalize the direction vector
                const normalizedDirectionX = directionX / length;
                const normalizedDirectionZ = directionZ / length;

                // Apply knockback using applyKnockback
                system.run(() => {
                    mod.applyKnockback(normalizedDirectionX * knockbackStrength, normalizedDirectionZ * knockbackStrength,knockbackStrength,2)
                })}
        }
    }
});


system.runInterval(() => {
    for (const entity of world.getDimension("overworld").getEntities()) {
        if (entity.typeId == "minecraft:fireworks_rocket") {
            const currentTime = Date.now();

            let options = { location: entity.location, maxDistance: 2, excludeTypes: ["minecraft:item", "minecraft:fireworks_rocket","minecraft:player"], excludeTags: ["Shoot"] };
            let checkEnemy = entity.dimension.getEntities(options);

            // Check for entities around the rocket
            if (checkEnemy.length > 0) {
                handleExplosion(entity);
                return
            }
            // Check for blocks around the rocket
            const offsets = [
                { x: 0, y: -0.2, z: 0 },
                { x: 0.2, y: 0, z: 0 },
                { x: -0.2, y: 0, z: 0 },
                { x: 0, y: 0, z: 0.2 },
                { x: 0, y: 0, z: -0.2 },
            ];

            for (const offset of offsets) {
                const checkUnder = {
                    x: entity.location.x + offset.x,
                    y: entity.location.y + offset.y,
                    z: entity.location.z + offset.z,
                };

                const blockHit = entity.dimension.getBlock(checkUnder);
                if (blockHit && blockHit.typeId !== "minecraft:air") {
                    handleExplosion(entity);
                    return;
                }
            }

            // Check for rocket velocity
            if (entity.getVelocity().x === 0 || entity.getVelocity().z === 0) {
                handleExplosion(entity);
                return;
            }
        }
    }
    
});

function handleExplosion(entity) {
    entity.dimension.spawnParticle("minecraft:knockback_roar_particle", entity.location);
    const loc = ` ${entity.location.x}  ${entity.location.y}  ${entity.location.z}`;
    entity.dimension.runCommand(`playsound firework.large_blast @a ${loc} 5 5`);
    entity.runCommand(`damage @e[r=6] 5`);
    entity.remove();
}
